@@ -17,15 +17,25 @@ local function serialize(value)
    if type(value) == "string" then
       return string.format ("%q", value)
    elseif type(value) == "table" then
-      local fields = {}
+      local serialized, fields = {}, {}
+
+      for i, element in ipairs (value) do
+	 table.insert (fields, string.format ("%s,", serialize(element)))
+
+	 serialized[i] = true
+      end
 
       for key, element in pairs (value) do
-	 table.insert (fields, string.format ("[%s] = %s,",
-					      serialize(key),
-					      serialize(element)))
+	 if not serialized[key] then
+	    table.insert (fields, string.format ("[%s] = %s,",
+						 serialize(key),
+						 serialize(element)))
+	 end
       end
       
       return "{" .. table.concat(fields) .. "}"
+   elseif type(value) == "number" then
+      return string.format ("%.17g", value)
    else
       return tostring(value)
    end
