@@ -17,9 +17,13 @@
 #ifndef _AIRPLANE_H_
 #define _AIRPLANE_H_
 
-#include "body.h"
+#include <ode/ode.h>
+#include "node.h"
 
 @interface Airplane: Node {
+    dBodyID body;
+    dJointID amotor, lmotor;
+    
     double controls[3];
     double area, span, chord;
 
