@@ -21,7 +21,9 @@
 
 @interface Annotation: Widget {
     id node;
+    float requested[2];
     double radius, angle, thickness;
+    int transformed;
 }
 
 @end
