@@ -21,7 +21,7 @@
 #include "transform.h"
 
 @interface Array: Transform {
-    int size, mold, spawn;
+    int size, spawn;
 }
 
 -(void) spawn: (int)m through: (int)n;
