@@ -227,10 +227,13 @@ static void calculate_sky_color(double azimuth, double elevation,
 	    }
 	}
     }
+
+    Y_0 = 1 / log(1 + Y_0);
     
     for(i = 0 ; i < width * height ; i += 1) {
 	double R, G, B;
 
+	/* Y_n = Y_0 * log(1 + Y[i]); */
 	Y_n = 1 - exp((0.05 * sintheta_s - 0.5) * Y[i]);
 	X = x[i] / y[i] * Y_n;
 	Z = (1 - x[i] - y[i]) / y[i] * Y_n;
