@@ -17,12 +17,12 @@
 #ifndef _ATMOSPHERE_H_
 #define _ATMOSPHERE_H_
 
-#include "node.h"
+#include "transform.h"
 
-@interface Atmosphere: Node {
-    GLuint skylight;
+@interface Atmosphere: Transform {
+    unsigned int skylight;
     
-    int size[2];
+    int size[2], set;
     float azimuth, elevation, turbidity, sunlight[3], direction[3];
     float rayleigh[3], mie;
 }
