@@ -21,7 +21,7 @@
 
 @interface Breaker: Node {
     id selected;
-    int state, target, reference;
+    int state, target;
 }
 
 -(void) link;
