@@ -164,6 +164,22 @@ static gboolean start_download (WebKitWebView *view,
     return TRUE;
 }
 
+gboolean decide_policy (WebKitWebView *webView,
+			WebKitWebFrame *frame,
+			WebKitNetworkRequest *request,
+			gchar *mimetype,
+			WebKitWebPolicyDecision *policy_decision,
+			gpointer user_data)
+{
+    if (webkit_web_view_can_show_mime_type(webView, mimetype)) {
+	webkit_web_policy_decision_use (policy_decision);
+    } else {
+	webkit_web_policy_decision_download (policy_decision);
+    }
+    
+    return TRUE;
+}
+
 static WebKitWebView* create_view (WebKitWebView *parent,
 				   WebKitWebFrame *frame,
 				   gpointer data)
@@ -201,6 +217,8 @@ static WebKitWebView* create_view (WebKitWebView *parent,
     		      G_CALLBACK (close_view), NULL);
     g_signal_connect (view, "download-requested",
     		      G_CALLBACK (start_download), NULL);
+    g_signal_connect (view, "mime-type-policy-decision-requested",
+    		      G_CALLBACK (decide_policy), NULL);
 
     gtk_container_add (GTK_CONTAINER (scrolled), GTK_WIDGET (view));
     gtk_container_add (GTK_CONTAINER (window), scrolled);
@@ -214,26 +232,22 @@ int main (int argc, char* argv[])
     GtkWidget *window;
     char *uri = NULL, *geometry = NULL;
     int i;
+    
+    gtk_init (&argc, &argv);
 
     for (i = 1 ; i < argc ; i += 1) {
-	if (!strcmp (argv[i], "-geometry") && i < argc - 1) {
+	if (!strcmp (argv[i], "--geometry") && i < argc - 1) {
 	    geometry = argv[i + 1];
 	    i += 1;
 	} else if (!uri && (argv[i][0] != '-')) {
 	    uri = argv[i];
 	} else {
-	    fprintf (stderr, "usage: %s [-geometry specification] [url]\n",
+	    fprintf (stderr, "usage: %s [--geometry specification] [url]\n",
 		     argv[0]);
 
 	    return 1;
 	}
     }
-    
-    gtk_init (&argc, &argv);
-    
-    if (!g_thread_supported ()) {
-        g_thread_init (NULL);
-    }
 
     view = create_view (NULL, NULL, NULL);
     window = gtk_widget_get_toplevel (GTK_WIDGET (view));
@@ -241,14 +255,14 @@ int main (int argc, char* argv[])
     g_signal_connect (window, "delete-event",
 		      G_CALLBACK (gtk_main_quit), NULL);
     
-    webkit_web_view_load_uri (view, uri ? uri : "http://localhost:29176");
-    
     if (geometry &&
 	!gtk_window_parse_geometry (GTK_WINDOW (window), geometry)) {
         fprintf (stderr,
 		 "%s: failed to parse geometry specification.\n",
 		 argv[0]);
     }
+    
+    webkit_web_view_load_uri (view, uri ? uri : "http://localhost:29176");
 
     show_window (view, NULL);
     
