@@ -18,10 +18,13 @@
 #define _CLOCK_H_
 
 #include <lua.h>
-#include "gauge.h"
+#include "widget.h"
 
-@interface Clock: Gauge {
+@interface Clock: Widget {
     id hands, markings;
+
+    int range[2], spacing[2], size;
+    double radius[2], spread[2], thickness, *reading;
 }
 
 @end
