@@ -40,6 +40,7 @@
 #include "contact.h"
 #include "conical.h"
 #include "polar.h"
+#include "gearing.h"
 
 #include "angular.h"
 #include "euler.h"
@@ -758,9 +759,9 @@ int luaopen_dynamics (lua_State *L)
     };	
     
     Class joints[] = {
-	[Slider class], [Universal class], [Hinge class], [Spherical class],
-	[Clamp class], [Contact class], [Conical class],
-	[Polar class]
+	[Slider class], [Universal class], [Hinge class],
+	[Spherical class], [Clamp class], [Contact class],
+	[Conical class], [Polar class], [Gearing class]
     };	
     
     Class motors[] = {
