@@ -112,7 +112,7 @@
 	[super traversePass: pass];
 
 	glDisable (GL_TEXTURE_2D);
-	glDisable(GL_BLEND);
+	/* glDisable(GL_BLEND); */
     }
 }
 
