@@ -20,12 +20,12 @@
 #include "node.h"
 
 @interface Fog: Node {
-    GLfloat offset, densityConstant[2], colorConstant[3];
+    float offset, density[2], color[3];
 }
 
--(float) offset;
--(float *) densityConstant;
--(float *) colorConstant;
+-(void) loadOffsetInto: (unsigned int) this;
+-(void) loadDensityInto: (unsigned int) this;
+-(void) loadColorInto: (unsigned int) this;
 
 @end
 
