@@ -22,19 +22,13 @@
 #include "joint.h"
 
 @interface Fourstroke: Joint {
-  dJointID transmission;
-  dBodyID flywheel;
-  dJointFeedback output;
-
-  double state[9];
-  double anchor[3], axis[3], tolerance;
-  double benchspeed, throttle, bypass;
-  double inertia, peak;
-  double displacement, intake[2];
-  double volumetric[4], thermal[3];
-  double exchange[2], friction[3];
-  double *gearbox, primary;
-  int cylinders, gear, gears, idle;
+    double state[9];
+    double anchor[3], axis[3], tolerance;
+    double benchspeed, throttle, bypass;
+    double displacement, intake[2];
+    double volumetric[4], thermal[3];
+    double exchange[2], friction[3];
+    int cylinders, idle, spark;
 }
 
 -(void) cycle;
