@@ -23,7 +23,6 @@
 
 #include "transform.h"
 #include "mustache.h"
-#include "stalker.h"
 #include "timer.h"
 #include "event.h"
 #include "top.h"
@@ -34,6 +33,7 @@
 #include "veneer.h"
 #include "cursor.h"
 #include "profiler.h"
+#include "meta.h"
 
 static char *decapitalize (char *s)
 {
@@ -157,11 +157,11 @@ int luaopen_frames (lua_State *L)
     int i;
     
     Class nodes[] = {
-	[Transform class], [Top class], [Stalker class],
+	[Transform class], [Top class],
 	[Timer class], [Gimbal class], [Observer class],
 	[Event class], [Mustache class], [Array class],
 	[Origin class], [Veneer class], [Cursor class],
-	[Node class], [Profiler class]
+	[Node class], [Profiler class], [Meta class]
     };	
 
     lua_newtable (L);
