@@ -18,9 +18,9 @@
 #define _GIMBAL_H_
 
 #include <lua.h>
-#include "node.h"
+#include "transform.h"
 
-@interface Gimbal: Node {
+@interface Gimbal: Transform {
 }
 @end
 
