@@ -23,11 +23,10 @@
 
 @interface Joint: Node {
     dJointID joint;
-    dJointFeedback feedback;
-    
-    id body;
+    dBodyID bodies[2];
+    dJointFeedback feedback;    
 
-    int attach, inverted, debug;
+    int attach, inverted, debug, explicit;
 }
 
 -(dJointID) joint;
