@@ -17,15 +17,14 @@
 #ifndef _LAMP_H_
 #define _LAMP_H_
 
-#include "node.h"
+#include "transform.h"
 
-@interface Lamp: Node {
-    GLuint intensity;
-    GLfloat ambience;
+@interface Lamp: Transform {
+    unsigned int intensity;
+    float ambience;
 }
 
--(GLfloat) ambience;
--(GLuint) intensity;
+-(void) setupTextureUnit;
 
 @end
 
