@@ -25,13 +25,20 @@
 #include "node.h"
 
 @interface Land: Node {
-    GLuint program, sampler, detailSampler, factor;
+    int palette;
+    
+    GLuint program, sampler, detailSampler[7], factor;
     GLuint turbidity, sunDirection, sunColor, eccentricity;
-    GLuint rayleigh, mie;
+    GLuint rayleigh, mie, power;
 
-    float albedo;
+    GLuint *detail;
+    double (*pigments)[8];
+
+    double albedo, separation;
 }
 
+-(void) rebuildWithPalette: (int)c;
+
 @end
 
 #endif
