@@ -22,7 +22,7 @@
 #include "joint.h"
 
 @interface Linear: Joint {
-    int degrees;
+    int degrees, relative[3];
     dVector3 axes[3];
     double motor[3][2], stops[3][2], hardness[3][2];
     double tolerance[3], bounce[3];
