@@ -29,15 +29,20 @@
 	glMultMatrixd ([self matrix]);
     
 	glUseProgramObjectARB(0);
+	
+	if (self->pattern != 0xffff) {
+	    glLineStipple(self->factor, self->pattern);
+	    glEnable (GL_LINE_STIPPLE);
+	}
 
 	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	glEnable(GL_DEPTH_TEST);
 	glEnable(GL_LINE_SMOOTH);
 	glEnable(GL_BLEND);
-	glLineWidth ([self width]);
+	glLineWidth (self->width);
 	glDepthMask (GL_FALSE);
 
-	glColor4fv([self color]);
+	glColor4dv(self->color);
 
 	glEnableClientState(GL_VERTEX_ARRAY);
 	glVertexPointer(3, GL_FLOAT, 0, self->vertices);
@@ -48,7 +53,8 @@
 	glDisable(GL_BLEND);
 	glDisable(GL_LINE_SMOOTH);
 	glDisable(GL_DEPTH_TEST);
-    
+    	glDisable (GL_LINE_STIPPLE);
+
 	glMatrixMode (GL_MODELVIEW);
 	glPopMatrix();
     }
