@@ -17,9 +17,9 @@
 #ifndef _LOOP_H_
 #define _LOOP_H_
 
-#include "source.h"
+#include "sample.h"
 
-@interface Loop: Source {
+@interface Loop: Sample {
 }
 @end
 
