@@ -48,9 +48,10 @@ static jmp_buf env;
 static ALCdevice *device;
 
 static int iterate = 1, frames, width, height, decorate;
-static int focus = LUA_REFNIL, defocus = LUA_REFNIL, delete = LUA_REFNIL;
+static int focus = LUA_REFNIL, defocus = LUA_REFNIL;
+static int configure = LUA_REFNIL, delete = LUA_REFNIL;
 static float canvas[3];
-static char *title, *phase;
+static char *title, *phase, *name = "techne", *class = "Techne";
 static const char *devicename;
 
 static double now, then, intervals[21], userinterval, interval = -1;
@@ -250,7 +251,8 @@ static int panic(lua_State *L)
     /* Don't let Lua terminate the process yet.
        There's cleaning up to do. */
     
-    /* longjmp (env, 1); */
+    lua_atpanic (_L, NULL);
+    longjmp (env, 1);
     
     return 0;
 }
@@ -361,6 +363,54 @@ static int children(lua_State *L)
     return 3;
 }
 
+static int joints_iterator(lua_State *L)
+{
+    id node;
+    int i, n;
+
+    node = *(id *)lua_touserdata(L, 1);
+    i = lua_tointeger (L, 2);
+    n = dBodyGetNumJoints([node body]);
+
+    if (i < n) {
+	node = dJointGetData(dBodyGetJoint([node body], i));
+	assert (node);
+    } else {
+	node = NULL;
+    }
+
+    if (node) {
+	/* Pop the old key and push the next. */
+	
+	lua_pop (L, 1);
+	lua_pushinteger (L, i + 1);
+	
+	/* Get the full userdata from the pointer. */
+	
+	lua_getfield (L, LUA_REGISTRYINDEX, "userdata");
+	lua_pushlightuserdata (L, node);
+	lua_gettable (L, -2);
+	lua_replace (L, -2);
+    } else {
+	/* We're done. */
+
+	lua_pop (L, 1);
+	lua_pushnil (L);
+	lua_pushnil (L);
+    }
+    
+    return 2;
+}
+
+static int joints(lua_State *L)
+{
+    lua_pushcfunction (L, joints_iterator);
+    lua_pushvalue (L, 1);
+    lua_pushinteger(L, 0);
+
+    return 3;
+}
+
 static int properties_iterator(lua_State *L)
 {
     id node;
@@ -859,6 +909,8 @@ static int graphics_index (lua_State *L)
 	luaX_pushunsignedshorts(L);
 
 	free(pixels);
+    } else if (!xstrcmp(k, "configure")) {
+	lua_rawgeti(L, LUA_REGISTRYINDEX, configure);
     } else if (!xstrcmp(k, "focus")) {
 	lua_rawgeti(L, LUA_REGISTRYINDEX, focus);
     } else if (!xstrcmp(k, "defocus")) {
@@ -917,6 +969,8 @@ static int graphics_newindex (lua_State *L)
 		    gdk_window_fullscreen (_WINDOW);
 		}
 	    }
+	    
+	    gdk_window_flush (_WINDOW);
 	}
 
 	/* Always flush when you're done. */
@@ -959,6 +1013,8 @@ static int graphics_newindex (lua_State *L)
 		glFlush();
 		glDrawBuffer (GL_BACK);
 	    }
+	    
+	    gdk_window_flush (_WINDOW);
 	}
 
 	hide = lua_toboolean (L, 3);
@@ -986,6 +1042,21 @@ static int graphics_newindex (lua_State *L)
 	new = gdk_cursor_new (cursor ? GDK_ARROW : GDK_BLANK_CURSOR);
 	gdk_window_set_cursor (_WINDOW, new);
 	gdk_cursor_destroy(new);
+    } else if (!xstrcmp(k, "pointer")) {
+	int x_w, y_w, x_p, y_p;
+	
+	lua_rawgeti (L, 3, 1);
+	x_p = lua_tointeger (L, -1);
+	
+	lua_rawgeti (L, 3, 2);
+	y_p = lua_tointeger (L, -1);
+
+	lua_pop (L, 2);
+	gdk_window_get_origin (_WINDOW, &x_w, &y_w);
+
+	gdk_display_warp_pointer (gdk_display_get_default(),
+				  gdk_screen_get_default(),
+				  x_p + x_w, y_p + y_w);
     } else if (!xstrcmp(k, "perspective")) {
 	GLdouble p[6];
 
@@ -1032,6 +1103,9 @@ static int graphics_newindex (lua_State *L)
 	}
 
  	glClearColor (canvas[0], canvas[1], canvas[2], 0);
+    } else if (!xstrcmp(k, "configure")) {
+	luaL_unref (L, LUA_REGISTRYINDEX, configure);
+	configure = luaL_ref (L, LUA_REGISTRYINDEX);
     } else if (!xstrcmp(k, "focus")) {
 	luaL_unref (L, LUA_REGISTRYINDEX, focus);
 	focus = luaL_ref (L, LUA_REGISTRYINDEX);
@@ -1077,6 +1151,8 @@ int main(int argc, char **argv)
     int parameters[] = {0, 0, 0, 0, 0};
 
     static struct option options[] = {
+	{"name", 1, 0, 'n'},
+	{"class", 1, 0, 'C'},
 	{"option", 1, 0, 'O'},
 	{"engineering", 0, 0, 'e'},
 	{"device", 1, 0, 'd'},
@@ -1137,6 +1213,9 @@ int main(int argc, char **argv)
     lua_pushcfunction (_L, children);
     lua_setfield (_L, LUA_GLOBALSINDEX, "children");
 
+    lua_pushcfunction (_L, joints);
+    lua_setfield (_L, LUA_GLOBALSINDEX, "constraints");
+
     lua_pushcfunction (_L, properties);
     lua_setfield (_L, LUA_GLOBALSINDEX, "properties");
     
@@ -1230,6 +1309,10 @@ int main(int argc, char **argv)
 	    /* Finally set the option value. */
 	    
 	    lua_settable (_L, -3);
+	} else if (option == 'n') {
+	    name = optarg;
+	} else if (option == 'C') {
+	    class = optarg;
 	} else if (option == 'e') {
 	    engineering = 1;
 	} else if (option == 'm') {
@@ -1325,7 +1408,12 @@ int main(int argc, char **argv)
                     "  -s BITS, --stencil=BITS           "
 		    "Set the stencil buffer depth.\n"
                     "  -z BITS, --depth=BITS             "
-		    "Set the Z-buffer depth.\n", argv[0]);
+		    "Set the Z-buffer depth.\n\n"
+		    "Windowing-system related options:\n"
+		    "  --name=NAME                       "
+		    "Set the application name.\n"
+		    "  --class=CLASS                     "
+		    "Set the application class.\n", argv[0]);
 
 	    exit(1);
 	} else {
@@ -1415,7 +1503,7 @@ int main(int argc, char **argv)
     if (!engineering) {
 	char *list[] = {
 	    "canvas", "close", "colorbuffer", "defocus", "extensions",
-	    "focus", "frames", "hide", "version", "window",
+	    "focus", "configure", "frames", "hide", "version", "window",
 	    "orthographic", "perspective", "renderer", "title", "cursor",
 	    "vendor", "grabinput"
 	};
@@ -1520,6 +1608,8 @@ int main(int argc, char **argv)
 
 	attributes.window_type = GDK_WINDOW_TOPLEVEL;
         attributes.wclass = GDK_INPUT_OUTPUT;
+	attributes.wmclass_name = name;
+	attributes.wmclass_class = class;
         attributes.colormap = gdk_gl_config_get_colormap(config);
         attributes.visual = gdk_gl_config_get_visual(config);
         attributes.width = 640;
@@ -1535,7 +1625,8 @@ int main(int argc, char **argv)
 
         _WINDOW = gdk_window_new(gdk_get_default_root_window(),
                                 &attributes, 
-                                GDK_WA_COLORMAP | GDK_WA_VISUAL);
+                                GDK_WA_COLORMAP | GDK_WA_VISUAL |
+				GDK_WA_WMCLASS);
 	
 	gdk_window_set_gl_capability(_WINDOW, config, NULL);
 	
@@ -1630,7 +1721,6 @@ int main(int argc, char **argv)
     sigaction (SIGQUIT, &action, NULL);
     sigaction (SIGILL, &action, NULL);
     sigaction (SIGABRT, &action, NULL);
-    sigaction (SIGKILL, &action, NULL);
     sigaction (SIGSEGV, &action, NULL);
     sigaction (SIGPIPE, &action, NULL);
     sigaction (SIGALRM, &action, NULL);
@@ -1723,9 +1813,6 @@ int main(int argc, char **argv)
 	    while ((event = gdk_event_get()) != NULL) {
 		int h;
 
-		/* Process the next event if it's any of
-		   our busyness. */
-	    
 		assert(event);
 	    
 		h = lua_gettop (_L);
@@ -1734,6 +1821,8 @@ int main(int argc, char **argv)
 		    glViewport (0, 0,
 				((GdkEventConfigure *)event)->width,
 				((GdkEventConfigure *)event)->height);
+		    
+		    lua_rawgeti (_L, LUA_REGISTRYINDEX, configure);
 		} else if (event->type == GDK_FOCUS_CHANGE) {
 		    if (((GdkEventFocus *)event)->in == TRUE) {
 			lua_rawgeti (_L, LUA_REGISTRYINDEX, focus);
@@ -1745,6 +1834,9 @@ int main(int argc, char **argv)
 		}
 
 		if (lua_gettop (_L) > h) {
+		    /* Process the next event if it's any of
+		       our busyness. */
+	    
 		    if (lua_isfunction (_L, -1)) {
 			luaX_call (_L, 0, 0);
 		    } else if (lua_istable (_L, -1)) {
@@ -1757,18 +1849,13 @@ int main(int argc, char **argv)
 		    } else {
 			lua_pop (_L, 1);
 		    }
+		} else {
+		    /* Otherwise pass it on to the node tree. */
+		    
+		    [root inputWithEvent: event];
 		}
-
-		/* Put it back for others to see. */
-
-		gdk_event_put (event);
+		
 		gdk_event_free (event);
-
-		[root input];
-
-		/* Remove the event from the queue. */
-	    
-		gdk_event_free (gdk_event_get ());
 	    }
 	}
 
@@ -1944,15 +2031,28 @@ int main(int argc, char **argv)
 	
 	frames += 1;
     }
+    
+    /* We're on are way out anyway so there's no
+       point catching signals past this point. */
+
+    action.sa_handler = SIG_DFL;
+    action.sa_flags = 0;
+    sigemptyset (&action.sa_mask);
+    
+    sigaction (SIGHUP, &action, NULL);
+    sigaction (SIGINT, &action, NULL);
+    sigaction (SIGQUIT, &action, NULL);
+    sigaction (SIGILL, &action, NULL);
+    sigaction (SIGABRT, &action, NULL);
+    sigaction (SIGSEGV, &action, NULL);
+    sigaction (SIGPIPE, &action, NULL);
+    sigaction (SIGALRM, &action, NULL);
+    sigaction (SIGTERM, &action, NULL);
 
     /* Unlink the whole scene. */
 
     [root toggle];
 
-    /* Finish cleaning up. */
-    
-    lua_close (_L);
-
     /* Destroy the audio context. */
     
     if (audio) {
@@ -1964,5 +2064,6 @@ int main(int argc, char **argv)
     }
     
     puts ("Adieu!");
-    return 0;
+
+    exit (EXIT_SUCCESS);
 }
