@@ -18,9 +18,9 @@
 #define _MUSTACHE_H_
 
 #include <lua.h>
-#include "node.h"
+#include "transform.h"
 
-@interface Mustache: Node {
+@interface Mustache: Transform {
 }
 @end
 
