@@ -422,6 +422,7 @@ static int respond (void *cls,
 	    MHD_add_response_header (response, "Server", "Techne/" VERSION);
 	    MHD_add_response_header (response, "Cache-Control", "no-store");
 	    /* MHD_add_response_header (response, "Connection", "close"); */
+	    
 	    MHD_add_response_header (response, "Content-Type",
 				     lua_tostring (_L, -2) ?
 				     lua_tostring (_L, -2) : "text/html");
@@ -896,21 +897,27 @@ static int constructpeer(lua_State *L)
 
 -(void) begin
 {
-    run();
+    do {
+	run();
+    } while (block);
 
     [super begin];
 }
 
 -(void) finish
 {
-    run();
+    do {
+	run();
+    } while (block);
 
     [super finish];
 }
 
 -(void) stepBy: (double)h at: (double)t
 {
-    run();
+    do {
+	run();
+    } while (block);
     
     [super stepBy: h at: t];
 }
