@@ -32,7 +32,7 @@
 -(void) stepBy: (double)h at: (double) t;
 -(void) toggle;
 -(void) begin;
--(void) input;
+-(void) inputWithEvent: (GdkEvent *) event;
 -(void) prepare;
 -(void) traversePass: (int)pass;
 -(void) finish;
