@@ -18,10 +18,9 @@
 #define _OBSERVER_H_
 
 #include <lua.h>
-#include "node.h"
+#include "transform.h"
 
-@interface Observer: Node {
-    GLfloat lookat[16];
+@interface Observer: Transform {
 }
 @end
 
