@@ -81,12 +81,17 @@
 
 	glUseProgramObjectARB(0);
 	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-    
+ 
+	if (self->pattern != 0xffff) {
+	    glLineStipple(self->factor, self->pattern);
+	    glEnable (GL_LINE_STIPPLE);
+	}
+   
 	glEnable(GL_DEPTH_TEST);
 	glEnable(GL_POLYGON_SMOOTH);
 	glEnable(GL_BLEND);
 
-	glColor4fv([self color]);
+	glColor4dv(self->color);
     
 	glBegin (GL_TRIANGLE_FAN);
 
@@ -102,7 +107,8 @@
 	glDisable(GL_BLEND);
 	glDisable(GL_POLYGON_SMOOTH);
 	glDisable(GL_DEPTH_TEST);
-    
+  	glDisable (GL_LINE_STIPPLE);
+  
 	glPopMatrix();
     }
     
