@@ -15,27 +15,9 @@
  */
 
 #include "racetrack.h"
+#include "ground.h"
 #include "wheel.h"
 
-static int callback (dGeomID TriMesh, dGeomID RefObject, int TriangleIndex)
-{
-    id object;
-
-    /* Don't collide the racetrack backup geom with the wheel backup geom. */
-    
-    if (dGeomGetClass (RefObject) == dCylinderClass) {
-	object = dGeomGetData(RefObject);
-    
-	if ([object isKindOf: [Wheel class]]) {
-	    return 0;
-	} else {
-	    return 1;
-	}
-    } else {
-	return 1;
-    }
-}
-
 static int test (double *segments, int index, int size, double tolerance,
 		 const dReal *q, dReal *n, dReal *d,
 		 double *longitude, double *latitude)
@@ -102,9 +84,11 @@ static int test (double *segments, int index, int size, double tolerance,
 		    cos(phi) * sin (psi) * sin (theta_0);
 		n[2] = cos(phi) * cos(psi);
 
-		*d = p_0[2] +
-		    u * g_0 + 0.5 * delta_g * u * u / S +
-		    v * e - q[2];
+		/* Project vertical depth onto normal. */
+		
+		*d = (p_0[2] +
+		      u * g_0 + 0.5 * delta_g * u * u / S +
+		      v * e - q[2]) * n[2];
 
 		if (longitude) {
 		    double u_0;
@@ -165,9 +149,11 @@ static int test (double *segments, int index, int size, double tolerance,
 		    cos(phi) * sin (psi) * sin (theta);
 		n[2] = cos(phi) * cos(psi);
 
-		*d = p_0[2] +
-		    u * g_0 + 0.5 * delta_g * u * u / S +
-		    v * e - q[2];
+		/* Project vertical depth onto normal. */
+		
+		*d = (p_0[2] +
+		      u * g_0 + 0.5 * delta_g * u * u / S +
+		      v * e - q[2]) * n[2];
 
 		if (longitude) {
 		    double u_0;
@@ -247,9 +233,11 @@ static int test (double *segments, int index, int size, double tolerance,
 			cos(phi) * sin (psi) * sin (theta);
 		    n[2] = cos(phi) * cos(psi);
 
-		    *d = p_0[2] +
-			u * g_0 + 0.5 * delta_g * u * u / S +
-			v * e - q[2];
+		    /* Project vertical depth onto normal. */
+		
+		    *d = (p_0[2] +
+			  u * g_0 + 0.5 * delta_g * u * u / S +
+			  v * e - q[2]) * n[2];
 
 		    if (longitude) {
 			double u_0;
@@ -285,11 +273,9 @@ static int test (double *segments, int index, int size, double tolerance,
     return -1;
 }
 
-static int collideWithWheel (dGeomID track,
-			     dGeomID wheel,
-			     int flags,
-			     dContactGeom *contact,
-			     int skip)
+static int collideTrackWithWheel (dGeomID track,
+				  dGeomID wheel,
+				  dContactGeom *contact)
 {
     struct wheeldata *wheeldata;
     struct trackdata *trackdata;
@@ -297,19 +283,37 @@ static int collideWithWheel (dGeomID track,
     const dReal *R_t, *r_t, *R_w, *r_w;
     dVector3 n = {0, 0, 1}, r, rprime;
     dMatrix3 R;
+
+    int segment;
     
     trackdata = dGeomGetClassData (track);
     wheeldata = dGeomGetClassData (wheel);
-	
+    
     r_w = dGeomGetPosition (wheel);
     R_w = dGeomGetRotation (wheel);
 	
     r_t = dGeomGetPosition (track);
     R_t = dGeomGetRotation (track);
 
+    /* Transform into track space. */
+    
     dOP (rprime, -, r_w, r_t);
     dMULTIPLY1_331 (r, R_t, rprime);
     dMULTIPLY1_333 (R, R_t, R_w);
+
+    /* Get an estimate of the surface normal by
+       testing the wheel center location. */
+    
+    segment = test (trackdata->segments,
+		    trackdata->last,
+		    trackdata->size,
+		    trackdata->tolerance,
+		    r, n, &wheeldata->contact.depth,
+		    NULL, NULL);
+	
+    if (segment < 0) {
+	return -1;
+    }
    
     wheeldata->contact.g1 = wheel;
     wheeldata->contact.g2 = track;
@@ -317,22 +321,34 @@ static int collideWithWheel (dGeomID track,
     wheeldata->axial[0] = R[1];
     wheeldata->axial[1] = R[5];
     wheeldata->axial[2] = R[9];
-
-    /* Get an estimate of the surface normal by
-       testing the wheel center location. */
-    
-    test (trackdata->segments,
-	  wheeldata->index,
-	  trackdata->size,
-	  trackdata->tolerance,
-	  r, n, &wheeldata->contact.depth,
-	  NULL, NULL);
 	
     dCROSS(wheeldata->longitudinal, =, wheeldata->axial, n);
     dCROSS(wheeldata->radial, =, wheeldata->axial, wheeldata->longitudinal);
     dCROSS(wheeldata->lateral, =, n, wheeldata->longitudinal);
 
+    /* Normalize both longitudinal and radial vectors.  (Thanks
+       to Simon Fendall for pointing out that the longitudinal
+       and lateral vectors need to be normalized as well. */
+    
+    dSafeNormalize3 (wheeldata->longitudinal);
     dSafeNormalize3 (wheeldata->radial);
+    dSafeNormalize3 (wheeldata->lateral);
+
+#if 0
+    {
+        Node *foo;
+
+        foo = (Node *)dGeomGetData (wheel);
+
+        if (foo->tag == 2) {
+	    printf ("%f, %f, %f, %f\n",
+		    dLENGTH(n),
+		    dLENGTH(wheeldata->axial),
+		    dLENGTH(wheeldata->longitudinal),
+		    dLENGTH(wheeldata->radial));
+        }
+    }
+#endif
 
     /* Calculate the contact point. */
 
@@ -348,18 +364,24 @@ static int collideWithWheel (dGeomID track,
 	r[2] + wheeldata->radial[2] * wheeldata->radii[0] -
 	n[2] * wheeldata->radii[1];
 
-    wheeldata->index = test (trackdata->segments,
-			     wheeldata->index,
-			     trackdata->size,
-			     trackdata->tolerance,
-			     wheeldata->contact.pos,
-			     wheeldata->contact.normal,
-			     &wheeldata->contact.depth,
-			     NULL, NULL);
+    segment = test (trackdata->segments,
+		    trackdata->last,
+		    trackdata->size,
+		    trackdata->tolerance,
+		    wheeldata->contact.pos,
+		    wheeldata->contact.normal,
+		    &wheeldata->contact.depth,
+		    NULL, NULL);
+	
+    if (segment < 0) {
+	return -1;
+    }
 
-    wheeldata->airborne = (wheeldata->index < 0) ||
-	(wheeldata->contact.depth < 0);
+    trackdata->last = segment;
+    wheeldata->airborne = (wheeldata->contact.depth < 0);
 
+    /* Transform back into the world. */
+    
     dMULTIPLY0_331 (rprime, R_t, wheeldata->contact.pos);
     dOP (wheeldata->contact.pos, +, rprime, r_t);
 
@@ -377,15 +399,299 @@ static int collideWithWheel (dGeomID track,
 
     dOPE (rprime, =, wheeldata->contact.normal);
     dMULTIPLY0_331 (wheeldata->contact.normal, R_t, rprime);
-    
-    /* printf ("%f, %f, %f\n", */
-    /* 	    wheeldata->contact.pos[0], */
-    /* 	    wheeldata->contact.pos[1], */
-    /* 	    wheeldata->contact.pos[2]); */
+ 	
+    /* { */
+    /*     Node *foo; */
+
+    /*     foo = (Node *)dGeomGetData (wheel); */
+    /*     if (foo->tag == 1) { */
+    /* 	    fprintf (stderr, "L: %f, %f, %f\n", wheeldata->longitudinal[0], wheeldata->longitudinal[1], wheeldata->longitudinal[2]); */
+    /* 	    fprintf (stderr, "Lat: %f, %f, %f\n", wheeldata->lateral[0], wheeldata->lateral[1], wheeldata->lateral[2]); */
+    /* 	    fprintf (stderr, "R: %f, %f, %f\n", wheeldata->radial[0], wheeldata->radial[1], wheeldata->radial[2]); */
+    /*     } */
+    /* } */
    
     return 0;
 }
 
+static int collideHeightfieldWithWheel (dGeomID field,
+					dGeomID wheel,
+					dContactGeom *contact,
+					void (*sampler)(int, int, double *, double *),
+					const int *l, const int d,
+					const double *s)
+{
+    struct wheeldata *wheeldata;
+
+    const dReal *R_t, *r_t, *R_w, *r_w;
+    dVector3 n = {0, 0, 1}, r, rprime;
+    dMatrix3 R;
+    double t_f, u_f, t_i, u_i;
+    
+    const int A[16 * 16] =
+	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+	 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0,
+	 -3, 0, 0, 3, 0, 0, 0, 0,-2, 0, 0,-1, 0, 0, 0, 0,
+	 2, 0, 0,-2, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0,
+	 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0,
+	 0, 0, 0, 0,-3, 0, 0, 3, 0, 0, 0, 0,-2, 0, 0,-1,
+	 0, 0, 0, 0, 2, 0, 0,-2, 0, 0, 0, 0, 1, 0, 0, 1,
+	 -3, 3, 0, 0,-2,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+	 0, 0, 0, 0, 0, 0, 0, 0,-3, 3, 0, 0,-2,-1, 0, 0,
+	 9,-9, 9,-9, 6, 3,-3,-6, 6,-6,-3, 3, 4, 2, 1, 2,
+	 -6, 6,-6, 6,-4,-2, 2, 4,-3, 3, 3,-3,-2,-1,-1,-2,
+	 2,-2, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+	 0, 0, 0, 0, 0, 0, 0, 0, 2,-2, 0, 0, 1, 1, 0, 0,
+	 -6, 6,-6, 6,-3,-3, 3, 3,-4, 4, 2,-2,-2,-2,-1,-1,
+	 4,-4, 4,-4, 2, 2,-2,-2, 2,-2,-2, 2, 1, 1, 1, 1};
+    
+    double y[16];
+    double x[16], c[16], y_i, y1_i, y2_i;
+    int i, j;
+
+    r_w = dGeomGetPosition (wheel);
+    R_w = dGeomGetRotation (wheel);
+	
+    r_t = dGeomGetPosition (field);
+    R_t = dGeomGetRotation (field);
+
+    /* Transform into field space. */
+    
+    dOP (rprime, -, r_w, r_t);
+    dMULTIPLY1_331 (r, R_t, rprime);
+    dMULTIPLY1_333 (R, R_t, R_w);
+
+    /* Calculate integer indices into the heighfield and
+       fractional remainders to interpolate to. Note that
+       the indices have to be offset to account for the
+       difference in the geoms origin versus the heightfield's
+       origin in memory. */
+    
+    t_f = modf(-r[0] / s[0] + 0.5 * (1 << d) * l[0], &t_i);
+    u_f = modf(r[2] / s[1] + 0.5 * (1 << d) * l[1], &u_i);
+
+    for (i = 0 ; i < 4 ; y[i] = NAN, i += 1);
+    for (i = 4 ; i < 16 ; y[i] = 0, i += 1);
+
+    /*********************
+
+    13     12     11     10
+    o.........o.........o.........o
+    .         .         .         .
+    .         .         .         .
+    .         .         .         .
+    .14       .3        .2       9.
+    o.........o.........o.........o
+    .         .         .         .
+    .         .t_f      .         .
+    .         .---x     .         .
+    .15       .0  |u_f  .1       8.
+    o.........o.........o.........o
+    .         .(ti, ui) .         .
+    .         .         .         .
+    .         .         .         .
+    .4        .5        .6       7.
+    o.........o.........o.........o
+    
+    **********************/
+
+    sampler (t_i + 0, u_i + 0, &y[0], NULL);
+    sampler (t_i + 1, u_i + 0, &y[1], NULL);
+    sampler (t_i + 1, u_i + 1, &y[2], NULL);
+    sampler (t_i + 0, u_i + 1, &y[3], NULL);
+
+    /* If any of these is still a NAN it means
+       we tried to index past the edge so just
+       return nothing. */
+	
+    for (i = 0 ; i < 4 ; i += 1) {
+	if (isnan (y[i])) {
+	    return -1;
+	}
+    }
+
+    /* Now sample the neighborhood. */
+
+    sampler (t_i - 1, u_i - 1, &y[4], NULL);
+    sampler (t_i + 0, u_i - 1, &y[5], NULL);
+    sampler (t_i + 1, u_i - 1, &y[6], NULL);
+    sampler (t_i + 2, u_i - 1, &y[7], NULL);
+    sampler (t_i + 2, u_i + 0, &y[8], NULL);
+    sampler (t_i + 2, u_i + 1, &y[9], NULL);
+    sampler (t_i + 2, u_i + 2, &y[10], NULL);
+    sampler (t_i + 1, u_i + 2, &y[11], NULL);
+    sampler (t_i + 0, u_i + 2, &y[12], NULL);
+    sampler (t_i - 1, u_i + 2, &y[13], NULL);
+    sampler (t_i - 1, u_i + 1, &y[14], NULL);
+    sampler (t_i - 1, u_i + 0, &y[15], NULL);
+
+    /* Now compute partial derivatives using Evans' method. */
+
+    x[0] = y[0]; x[1] = y[1];
+    x[2] = y[2]; x[3] = y[3];
+	
+    x[4] = (y[1] + y[2] + y[6] - y[15] - y[14] - y[4]) / 6;
+    x[5] = (y[8] + y[9] + y[7] - y[0] - y[3] - y[5]) / 6;
+    x[6] = (y[9] + y[10] + y[8] - y[3] - y[12] - y[0]) / 6;
+    x[7] = (y[2] + y[11] + y[1] - y[14] - y[13] - y[15]) / 6;
+
+    x[8] = (y[3] + y[14] + y[2] - y[5] - y[4] - y[6]) / 6;
+    x[9] = (y[2] + y[3] + y[9] - y[6] - y[5] - y[7]) / 6;
+    x[10] = (y[11] + y[12] + y[10] - y[1] - y[0] - y[8]) / 6;
+    x[11] = (y[12] + y[13] + y[11] - y[0] - y[15] - y[1]) / 6;
+
+    x[12] = 0.25 * (y[2] + y[4] - y[6] - y[14]);
+    x[13] = 0.25 * (y[9] + y[5] - y[7] - y[3]);	
+    x[14] = 0.25 * (y[10] + y[0] - y[8] - y[12]);	
+    x[15] = 0.25 * (y[11] + y[15] - y[1] - y[13]);
+
+    /* Calculate the coefficients for the bicubic spline
+       and interpolate.  This is according to 'Numerical
+       recipes, 3rd Edition, pp. 136' . */
+    
+    for (i = 0 ; i < 16 ; i += 1) {
+	c[i] = 0;
+	    
+	for (j = 0 ; j < 16 ; j += 1) {
+	    c[i] += A[i * 16 + j] * x[j];
+	}
+    }
+    
+    for (i = 3, y_i = 0, y1_i = 0, y2_i = 0;
+	 i >= 0 ;
+	 y_i = t_f * y_i + ((c[i * 4 + 3] * u_f + c[i * 4 + 2]) * u_f + c[i * 4 + 1]) * u_f + c[i * 4 + 0],
+	     y2_i = t_f * y2_i + (3.0 * c[i * 4 + 3] * u_f + 2.0 * c[i * 4 + 2]) * u_f + c[i * 4 + 1],
+	     y1_i = u_f * y1_i + (3.0 * c[3 * 4 + i] * t_f + 2.0 * c[2 * 4 + i]) * t_f + c[1 * 4 + i],
+	     i -= 1);
+
+    /* Calculate and normalize the
+       contact normal vector. */
+    
+    n[0] = -y1_i / s[0];
+    n[1] = 1;
+    n[2] = -y2_i / s[1];
+
+    dSafeNormalize3(n);
+
+    /* Now proceed as in the racetrack collision
+       handling routine. */
+    
+    wheeldata = dGeomGetClassData (wheel);
+    wheeldata->contact.g1 = wheel;
+    wheeldata->contact.g2 = field;
+    
+    wheeldata->axial[0] = R[1];
+    wheeldata->axial[1] = R[5];
+    wheeldata->axial[2] = R[9];
+
+    dCROSS(wheeldata->longitudinal, =, wheeldata->axial, n);
+    dCROSS(wheeldata->radial, =, wheeldata->axial, wheeldata->longitudinal);
+    dCROSS(wheeldata->lateral, =, n, wheeldata->longitudinal);
+
+    dSafeNormalize3 (wheeldata->radial);
+
+    wheeldata->contact.pos[0] =
+	r[0] + wheeldata->radial[0] * wheeldata->radii[0] -
+	n[0] * wheeldata->radii[1];
+	
+    wheeldata->contact.pos[1] =
+	r[1] + wheeldata->radial[1] * wheeldata->radii[0] -
+	n[1] * wheeldata->radii[1];
+	
+    wheeldata->contact.pos[2] =
+	r[2] + wheeldata->radial[2] * wheeldata->radii[0] -
+	n[2] * wheeldata->radii[1];
+
+    wheeldata->contact.normal[0] = n[0];
+    wheeldata->contact.normal[1] = n[1];
+    wheeldata->contact.normal[2] = n[2];
+
+    /* Project the vertical penetration depth onto the normal
+       to get the depth along the normal. */
+    
+    wheeldata->contact.depth = (y_i - wheeldata->contact.pos[1]) * n[1];
+    wheeldata->airborne = (wheeldata->contact.depth < 0);
+
+    /* Transform back into the world. */
+    
+    dMULTIPLY0_331 (rprime, R_t, wheeldata->contact.pos);
+    dOP (wheeldata->contact.pos, +, rprime, r_t);
+
+    dOPE (rprime, =, wheeldata->axial);
+    dMULTIPLY0_331 (wheeldata->axial, R_t, rprime);
+
+    dOPE (rprime, =, wheeldata->lateral);
+    dMULTIPLY0_331 (wheeldata->lateral, R_t, rprime);
+
+    dOPE (rprime, =, wheeldata->longitudinal);
+    dMULTIPLY0_331 (wheeldata->longitudinal, R_t, rprime);
+
+    dOPE (rprime, =, wheeldata->radial);
+    dMULTIPLY0_331 (wheeldata->radial, R_t, rprime);
+
+    dOPE (rprime, =, wheeldata->contact.normal);
+    dMULTIPLY0_331 (wheeldata->contact.normal, R_t, rprime);
+
+#if 0
+    {
+        Node *foo;
+
+        foo = (Node *)dGeomGetData (wheel);
+
+        if (foo->tag == 2) {
+    	    /* { */
+    	    /* 	double y_il = ((1 - t_f) * y[0] + t_f * y[1]) * (1 - u_f) + */
+    	    /* 	    ((1 - t_f) * y[3] + t_f * y[2]) * u_f; */
+		
+    	    /* 	printf ("%f, %f\n", fabs(y_i - y_il), wheeldata->contact.depth); */
+    	    /* } */
+	    
+    	    /* printf ("nn %.5f, %.5f, %.5f\n", */
+    	    /* 	    wheeldata->contact.normal[0], */
+    	    /* 	    wheeldata->contact.normal[1], */
+    	    /* 	    wheeldata->contact.normal[2]); */
+	
+    	    /* printf ("pp %.5f, %.5f, %.5f\n", */
+    	    /* 	    wheeldata->contact.pos[0], */
+    	    /* 	    wheeldata->contact.pos[1], */
+    	    /* 	    wheeldata->contact.pos[2]); */
+    	    /* printf ("N %f\n", dLENGTH(wheeldata->contact.normal)); */
+    	    /* printf ("dd %f\n", wheeldata->contact.depth); */
+        }
+    }
+#endif
+  
+    return 0;
+}
+
+static int collideWithWheel (dGeomID track,
+			     dGeomID wheel,
+			     int flags,
+			     dContactGeom *contact,
+			     int skip)
+{
+    struct trackdata *trackdata;
+    struct wheeldata *wheeldata;
+
+    trackdata = dGeomGetClassData (track);
+    wheeldata = dGeomGetClassData (wheel);
+    
+    if (collideTrackWithWheel (track, wheel, contact) < 0) {
+	if (!trackdata->field ||
+	    collideHeightfieldWithWheel (trackdata->field,
+					 wheel, contact,
+					 trackdata->sampler,
+					 trackdata->tiles,
+					 trackdata->depth,
+					 trackdata->resolution) < 0) {
+	    wheeldata->airborne = 1;
+	}
+    }
+
+    return 0;
+}
+
 static dColliderFn * getCollider (int num)
 {
     if (num == dWheelClass) {
@@ -395,12 +701,75 @@ static dColliderFn * getCollider (int num)
     }
 }
 
+static int sampler_tostring(lua_State *L)
+{
+    lua_pushstring(L, "Sampler");
+   
+    return 1;
+}
+
+static int sampler_index(lua_State *L)
+{
+    struct trackdata *data;
+    
+    if (lua_istable (L, 2)) {
+	double n[3], d, u, v;
+	double p[3] = {0, 0, 0};
+	int s;
+
+	/* Get the track data. */
+	
+	lua_getmetatable (L, 1);
+	lua_pushstring (L, "__data");
+	lua_gettable (L, -2);
+	data = lua_touserdata (L, -1);
+	lua_pop (L, 2);
+
+	/* Get the sampling point. */
+	
+	lua_rawgeti (L, 2, 1);
+	p[0] = lua_tonumber (L, -1);
+
+	lua_rawgeti (L, 2, 2);
+	p[1] = lua_tonumber (L, -1);
+
+	lua_pop (L, 2);
+
+	/* Sample the track. */
+	
+	s = test (data->segments, 0,
+		  data->size, data->tolerance,
+		  p, n, &d, &u, &v);
+	
+	if (s >= 0) {
+	    lua_newtable (L);
+	    lua_pushinteger (L, s);
+	    lua_rawseti (L, -2, 1);
+	    
+	    lua_pushnumber (L, u);
+	    lua_rawseti (L, -2, 2);
+	    
+	    lua_pushnumber (L, v);
+	    lua_rawseti (L, -2, 3);
+	    
+	    lua_pushnumber (L, d);
+	    lua_rawseti (L, -2, 4);	    
+	} else {
+	    lua_pushboolean (L, 0);
+	}
+    } else {
+	lua_pushnil (L);
+    }
+
+    return 1;
+}
+
 @implementation Racetrack
 
 -(Racetrack *)init
 {
     char *list[] = {
-	"tesselation", "scale"
+	"tessellation", "scale", "sampler"
     };
     
     struct trackdata *data;
@@ -415,10 +784,6 @@ static dColliderFn * getCollider (int num)
       
 	dTrackClass = dCreateGeomClass (&class);
     }
-    
-    self->meshdata = dGeomTriMeshDataCreate ();
-    self->mesh = dCreateTriMesh (NULL, self->meshdata, callback, NULL, NULL);
-    dGeomSetData (self->mesh, self);
 
     self->geom = dCreateGeom (dTrackClass);
     dGeomSetData (self->geom, self);
@@ -426,16 +791,15 @@ static dColliderFn * getCollider (int num)
     self->vertices = NULL;
     self->normals = NULL;
     self->uv = NULL;
-    self->indices = NULL;
     self->length = 0;
     self->dirty = 0;
 
     self->scale[0] = 1;
     self->scale[1] = 1;
     
-    self->tesselation[0] = 2 * M_PI / 32;
-    self->tesselation[1] = 0.01 / 10;
-    self->tesselation[2] = 0.1 / 10;
+    self->tessellation[0] = 2 * M_PI / 32;
+    self->tessellation[1] = 0.01 / 10;
+    self->tessellation[2] = 0.1 / 10;
 
     data = dGeomGetClassData (self->geom);
 
@@ -445,35 +809,44 @@ static dColliderFn * getCollider (int num)
     data->segments = NULL;
     data->size = 0;
     data->tolerance = 0.01 / 10;
+    data->last = 0;
+    
+    data->tiles = NULL;
+    data->depth = 0;
+    data->resolution = NULL;
+    data->sampler = NULL;
 
     return self;
 }
 
--(void) fasten
+-(void) meetSibling: (id)sibling
 {
-    [super fasten];
+    struct trackdata *data;
 
-    dGeomSetBody (self->mesh, NULL);
+    if ([sibling isKindOf: [Ground class]]) {
+	data = dGeomGetClassData (self->geom);
+
+	data->sampler = [sibling sampler];
+	data->field = [sibling geom];
+	data->tiles = [sibling size];
+	data->depth = [sibling depth];
+	data->resolution = [sibling resolution];
+    }
 }
 
--(void) release
+-(void) missSibling: (Item *)sibling
 {
-    [super release];
+    struct trackdata *data;
 
-    dGeomSetBody (self->mesh, self->body);
-}
+    if ([sibling isKindOf: [Ground class]]) {
+	data = dGeomGetClassData (self->geom);
 
--(void) insertInto: (dSpaceID) new
-{
-    if (self->space) {
-    	dSpaceRemove (self->space, self->mesh);
+	data->sampler = NULL;
+	data->field = NULL;
+	data->tiles = NULL;
+	data->depth = 0;
+	data->resolution = NULL;
     }
-    
-    if (new) {
-    	dSpaceAdd (new, self->mesh);
-    }
-
-    [super insertInto: new];
 }
 
 -(void) update
@@ -554,7 +927,7 @@ static dColliderFn * getCollider (int num)
 	    double dv[2], u;
 	    int m;
 
-	    m = fmax(ceil (sqrt(fabs (delta_g) * S / 8 / self->tesselation[2])), 1);
+	    m = fmax(ceil (sqrt(fabs (delta_g) * S / 8 / self->tessellation[2])), 1);
 
 	    /* printf ("++ %d, %f, %f\n", i, e_0, delta_e); */
 		
@@ -633,8 +1006,8 @@ static dColliderFn * getCollider (int num)
 
 	    r = 1 / kappa_0;
 
-	    m = fmax(ceil(S / self->tesselation[0]),
-		     ceil (sqrt(fabs (delta_g) * S / 8 / self->tesselation[2])));
+	    m = fmax(ceil(S / self->tessellation[0]),
+		     ceil (sqrt(fabs (delta_g) * S / 8 / self->tessellation[2])));
 	    m = m > 0 ? m : 1;
 		
 	    c[0] = p_0[0] - r * sin(theta_0);
@@ -720,10 +1093,10 @@ static dColliderFn * getCollider (int num)
 	    double r, c[2], u;
 	    int m, n, k;
 
-	    m = (int)fmax(ceil(fabs(delta_kappa) / self->tesselation[1]), 1);
+	    m = (int)fmax(ceil(fabs(delta_kappa) / self->tessellation[1]), 1);
 
-	    n = fmax(ceil(S / m / self->tesselation[0]),
-		     ceil (sqrt(fabs (delta_g * S) / m / m / 8 / self->tesselation[2])));
+	    n = fmax(ceil(S / m / self->tessellation[0]),
+		     ceil (sqrt(fabs (delta_g * S) / m / m / 8 / self->tessellation[2])));
 	    n = n > 0 ? n : 1;
 	    /* printf ("* %d, %d, %d\n", i, m, n); */
 		
@@ -826,33 +1199,6 @@ static dColliderFn * getCollider (int num)
     }
 
     /* printf ("!!! %f, %f, %f,  %f\n", p_0[0], p_0[1], p_0[2], theta_0 / M_PI); */
-
-    /* Write the indices. */
-
-    self->indices = realloc (self->indices, 6 * (self->length / 2 - 1) * sizeof (dTriIndex));
-
-    for (i = 0 ; i < self->length / 2 - 1 ; i += 1) {
-	self->indices[6 * i] = 2 * i;
-	self->indices[6 * i + 1] = 2 * i + 1;
-	self->indices[6 * i + 2] = 2 * i + 2;
-	self->indices[6 * i + 3] = 2 * i + 2;
-	self->indices[6 * i + 4] = 2 * i + 1;
-	self->indices[6 * i + 5] = 2 * i + 3;
-    }
-
-    /* Build the mesh data. */
-    
-    dGeomTriMeshSetData (self->mesh, NULL);
-    dGeomTriMeshDataDestroy (self->meshdata);
-    
-    self->meshdata = dGeomTriMeshDataCreate ();
-
-    dGeomTriMeshDataBuildDouble (self->meshdata,
-			   self->vertices, 3 * sizeof(double), self->length,
-			   self->indices, 6 * (self->length / 2 - 1),
-			   3 * sizeof (dTriIndex));
-
-    dGeomTriMeshSetData (self->mesh, self->meshdata);
     
     self->dirty = 0;
 }
@@ -866,54 +1212,14 @@ static dColliderFn * getCollider (int num)
     }
 }
 
-/* -(void) transform */
-/* { */
-/*     dGeomTriMeshDataSet(self->meshdata, */
-/* 			TRIMESH_LAST_TRANSFORMATION, */
-/* 			(void *) matrix); */
-/*     [super transform]; */
-/* } */
-
 -(void) get
 {
     struct trackdata *data;
-
-    const char *k;
     int i, j;
-    
+
     data = dGeomGetClassData (self->geom);
-    k = lua_tostring (_L, 2);
 
-    if (lua_istable (_L, 2)) {
-	double n[3], d, u, v;
-	double p[3] = {0, 0, 0};
-	
-	lua_rawgeti (_L, 2, 1);
-	p[0] = lua_tonumber (_L, -1);
-
-	lua_rawgeti (_L, 2, 2);
-	p[1] = lua_tonumber (_L, -1);
-
-	lua_pop (_L, 2);
-
-	if (self->dirty) {
-	    [self update];
-	}
-
-	if (test (data->segments, 0, data->size, data->tolerance, p, n, &d, &u, &v) >= 0) {
-	    lua_newtable (_L);
-	    lua_pushnumber (_L, u);
-	    lua_rawseti (_L, -2, 1);
-	    
-	    lua_pushnumber (_L, v);
-	    lua_rawseti (_L, -2, 2);
-	    
-	    lua_pushnumber (_L, d);
-	    lua_rawseti (_L, -2, 3);	    
-	} else {
-	    lua_pushboolean (_L, 0);
-	}
-    } else if (lua_isnumber (_L, 2)) {
+    if (lua_type (_L, 2) == LUA_TNUMBER) {
 	j = lua_tonumber (_L, 2) - 1;
 	
 	lua_newtable (_L);
@@ -922,52 +1228,72 @@ static dColliderFn * getCollider (int num)
             lua_pushnumber (_L, data->segments[10 * j + i]);
             lua_rawseti (_L, -2, i + 1);
         }
-    } else if (!xstrcmp(k, "vertices")) {
-	if (self->dirty) {
-	    [self update];
-	}
-
-	lua_newtable (_L);
-
-	for (i = 0 ; i < 3 * self->length ; i += 1) {
-	    lua_pushnumber (_L, self->vertices[i]);
-	    lua_rawseti (_L, -2, i + 1);
-	}
-    } else if (!xstrcmp(k, "scale")) {
-	lua_newtable (_L);
-	
-	lua_pushnumber (_L, self->scale[0]);
-	lua_rawseti (_L, 3, 1);
-	
-	lua_pushnumber (_L, self->scale[1]);
-	lua_rawseti (_L, 3, 2);
-    } else if (!xstrcmp(k, "tesselation")) {
-	lua_newtable (_L);
-	
-	lua_pushnumber (_L, self->tesselation[0]);
-	lua_rawseti (_L, 3, 1);
-	
-	lua_pushnumber (_L, self->tesselation[1]);
-	lua_rawseti (_L, 3, 2);
-	
-	lua_pushnumber (_L, self->tesselation[2]);
-	lua_rawseti (_L, 3, 3);
     } else {
-	[super get];
+	const char *k;
+    
+	k = lua_tostring (_L, 2);
+	
+	if (!xstrcmp(k, "sampler")) {
+	    lua_newtable (_L);
+
+	    lua_newtable (_L);
+	    lua_pushstring(_L, "__track");
+	    lua_pushvalue (_L, 1);
+	    lua_settable(_L, -3);
+	    lua_pushstring(_L, "__data");
+	    lua_pushlightuserdata (_L, data);
+	    lua_settable(_L, -3);
+	    lua_pushstring(_L, "__index");
+	    lua_pushcfunction(_L, (lua_CFunction)sampler_index);
+	    lua_settable(_L, -3);
+	    lua_pushstring(_L, "__tostring");
+	    lua_pushcfunction(_L, (lua_CFunction)sampler_tostring);
+	    lua_settable(_L, -3);
+	    lua_setmetatable(_L, -2);	
+	} else if (!xstrcmp(k, "vertices")) {
+	    if (self->dirty) {
+		[self update];
+	    }
+
+	    lua_newtable (_L);
+
+	    for (i = 0 ; i < 3 * self->length ; i += 1) {
+		lua_pushnumber (_L, self->vertices[i]);
+		lua_rawseti (_L, -2, i + 1);
+	    }
+	} else if (!xstrcmp(k, "scale")) {
+	    lua_newtable (_L);
+	
+	    lua_pushnumber (_L, self->scale[0]);
+	    lua_rawseti (_L, 3, 1);
+	
+	    lua_pushnumber (_L, self->scale[1]);
+	    lua_rawseti (_L, 3, 2);
+	} else if (!xstrcmp(k, "tessellation")) {
+	    lua_newtable (_L);
+	
+	    lua_pushnumber (_L, self->tessellation[0]);
+	    lua_rawseti (_L, 3, 1);
+	
+	    lua_pushnumber (_L, self->tessellation[1]);
+	    lua_rawseti (_L, 3, 2);
+	
+	    lua_pushnumber (_L, self->tessellation[2]);
+	    lua_rawseti (_L, 3, 3);
+	} else {
+	    [super get];
+	}
     }
 }
 
 -(void) set
 {
     struct trackdata *data;
-
-    const char *k;
     int i, j;
 
     data = dGeomGetClassData (self->geom);
-    k = lua_tostring (_L, 2);
 
-    if (lua_isnumber (_L, 2)) {
+    if (lua_type (_L, 2) == LUA_TNUMBER) {
         if(lua_istable (_L, 3)) {
 	    j = lua_tonumber (_L, 2);
 
@@ -986,29 +1312,43 @@ static dColliderFn * getCollider (int num)
 
 	    self->dirty = 1;
         }
-    } else if (!xstrcmp(k, "scale")) {
-	lua_rawgeti (_L, 3, 1);
-	self->scale[0] = lua_tonumber (_L, -1);
-	
-	lua_rawgeti (_L, 3, 2);
-	self->scale[1] = lua_tonumber (_L, -1);
-    } else if (!xstrcmp(k, "tesselation")) {
-	lua_rawgeti (_L, 3, 1);
-	self->tesselation[0] = lua_tonumber (_L, -1);
-	
-	lua_rawgeti (_L, 3, 2);
-	self->tesselation[1] = lua_tonumber (_L, -1);
-	data->tolerance = lua_tonumber (_L, -1);
-	
-	lua_rawgeti (_L, 3, 3);
-	self->tesselation[2] = lua_tonumber (_L, -1);
-
-	lua_pop (_L, 3);
     } else {
-	[super set];
+	const char *k;
+	k = lua_tostring (_L, 2);
+	
+	if (!xstrcmp(k, "scale")) {
+	    lua_rawgeti (_L, 3, 1);
+	    self->scale[0] = lua_tonumber (_L, -1);
+	
+	    lua_rawgeti (_L, 3, 2);
+	    self->scale[1] = lua_tonumber (_L, -1);
+	} else if (!xstrcmp(k, "tessellation")) {
+	    lua_rawgeti (_L, 3, 1);
+	    self->tessellation[0] = lua_tonumber (_L, -1);
+	
+	    lua_rawgeti (_L, 3, 2);
+	    self->tessellation[1] = lua_tonumber (_L, -1);
+	    data->tolerance = lua_tonumber (_L, -1);
+	
+	    lua_rawgeti (_L, 3, 3);
+	    self->tessellation[2] = lua_tonumber (_L, -1);
+
+	    lua_pop (_L, 3);
+	} else {
+	    [super set];
+	}
     }
 }
 
+-(int) length
+{
+    struct trackdata *data;
+
+    data = dGeomGetClassData (self->geom);
+    
+    return data->size;
+}
+
 -(void) free
 {
     struct trackdata *data;
@@ -1023,28 +1363,27 @@ static dColliderFn * getCollider (int num)
 	free (self->vertices);
 	free (self->normals);
 	free (self->uv);
-	free (self->indices);
     }
 
-    dGeomTriMeshDataDestroy (self->meshdata);
-    dGeomDestroy (self->mesh);
-
     [super free];
 }
 
 -(void) traversePass: (int)pass
 {
-   if (pass == 1) {
+   if (pass == 2 && self->debug) {
+	glUseProgramObjectARB(0);
+
 	glMatrixMode (GL_MODELVIEW);
 	glPushMatrix();
 	glMultMatrixd ([self matrix]);
 
-	glEnable (GL_DEPTH_TEST);
+	/* glEnable (GL_DEPTH_TEST); */
 	glEnable (GL_CULL_FACE);
    
-	/* glPolygonMode (GL_FRONT_AND_BACK, GL_LINE); */
+	glPolygonMode (GL_FRONT_AND_BACK, GL_LINE);
  
-	glClientActiveTexture (GL_TEXTURE0);
+	glColor3f (1, 0, 0);
+	glLineWidth (1);
 
 	glEnableClientState(GL_VERTEX_ARRAY);
 	glEnableClientState(GL_NORMAL_ARRAY);
