@@ -20,23 +20,30 @@
 #include "body.h"
 
 struct trackdata {
+    /* Tarmac. */
+
     double *segments, tolerance;
-    int size;
+    int size, last;
+
+    /* Terrain. */
+
+    dGeomID field;
+    const int *tiles;
+    int depth;
+    const double *resolution;
+    void (*sampler)(int, int, double *, double *);
 };
 
 int dTrackClass;
 
 @interface Racetrack: Body {
-    dGeomID mesh;
-    dTriMeshDataID meshdata;
-    double *vertices, *uv, *normals;
-    dTriIndex *indices;
-    
-    double tesselation[3], scale[2];
+    double *vertices, *uv, *normals;    
+    double tessellation[3], scale[2];
     int length, dirty;
 }
 
 -(void) update;
+-(int) length;
 
 @end
 
