@@ -151,7 +151,7 @@ int luaopen_shading (lua_State *L)
 	[Fresnel class], [Anisotropic class],
 	[Prototype class], [Glossy class],
 	[Cook class], [Sprites class],
-	[Cloth class]
+	[Cloth class],
     };	
     
     /* And the surface node constructors. */
