@@ -37,19 +37,12 @@
     self->color[2] = 1;
     self->color[3] = 1;
 
+    self->factor = 1;
+    self->pattern = 0xffff;
+
     return self;
 }
 
--(GLfloat *)color
-{
-    return self->color;
-}
-
--(float) width
-{
-    return self->width;
-}
-
 -(int) size
 {
     return self->size;
@@ -75,6 +68,12 @@
 	lua_pushnumber (_L, self->width);
     } else if (!xstrcmp(k, "opacity")) {
 	lua_pushnumber (_L, self->color[3]);
+    } else if (!xstrcmp(k, "stipple")) {
+	lua_newtable (_L);
+	lua_pushnumber (_L, self->factor);
+	lua_rawseti (_L, -2, 1);
+	lua_pushinteger (_L, self->pattern);
+	lua_rawseti (_L, -2, 2);
     } else if (!xstrcmp(k, "endpoint")) {
 	lua_pushnumber (_L, self->size);
 	lua_replace (_L, 2);
@@ -115,6 +114,18 @@
                 lua_pop (_L, 1);
             }
         }
+    } else if (!xstrcmp(k, "stipple")) {
+        if(lua_istable (_L, 3)) {
+	    lua_rawgeti (_L, 3, 1);
+	    self->factor = lua_tonumber (_L, -1);
+	    lua_pop (_L, 1);
+	    
+	    lua_rawgeti (_L, 3, 2);
+	    self->pattern = lua_tointeger (_L, -1);
+	    lua_pop (_L, 1);
+	} else {
+	    self->pattern = 0xffff;
+	}
     } else if (!xstrcmp(k, "width")) {
 	self->width = lua_tonumber (_L, 3);
     } else if (!xstrcmp(k, "opacity")) {
@@ -142,6 +153,8 @@
     if (self->vertices) {
 	free (self->vertices);
     }
+
+    [super free];
 }
 
 @end
