@@ -20,13 +20,11 @@
 #include "transform.h"
 
 @interface Shape: Transform {
-    GLfloat width, color[4];
-    GLfloat *vertices;
-    int size;
+    double width, color[4], factor;
+    float *vertices;
+    int size, pattern;
 }
 
--(GLfloat *)color;
--(GLfloat) width;
 -(int) size;
 
 @end
