@@ -27,6 +27,7 @@
 #include "halo.h"
 #include "pie.h"
 #include "path.h"
+#include "polygon.h"
 
 static char *decapitalize (char *s)
 {
@@ -150,7 +151,7 @@ int luaopen_shapes (lua_State *L)
     
     Class nodes[] = {
 	[Line class], [Lines class], [Points class], [Dashed class],
-	[Halo class], [Pie class], [Path class]
+	[Halo class], [Pie class], [Path class], [Polygon class]
     };	
 
     lua_newtable (L);
