@@ -79,6 +79,10 @@ static void callhooks (lua_State *L, void *key, int reference)
 -(void) free
 {
      alDeleteSources (1, &self->index);
+
+     luaL_unref (_L, LUA_REGISTRYINDEX, self->finished);
+
+     [super free];
 }
 
 -(ALint) index
