@@ -290,6 +290,19 @@
 	    }
 	}
     } else if (!xstrcmp(k, "stops")) {
+	for(j = 0 ; j < 3 ; j += 1) {
+	    /* Resetting the self->stops makes sure that lo remains
+	       smaller than hi between calls. */
+
+	    dJointSetAMotorParam ([self amotor],
+				  dParamLoStop + dParamGroup * j,
+				  -dInfinity);
+		    
+	    dJointSetAMotorParam ([self amotor],
+				  dParamHiStop + dParamGroup * j,
+				  dInfinity);
+	}
+	    
         if(lua_istable (_L, 3)) {
 	    for(j = 0 ; j < 3 ; j += 1) {
 		lua_rawgeti (_L, 3, j + 1);
@@ -318,18 +331,6 @@
 		    lua_rawgeti (_L, -1, 3);
 		    bounce[j] = lua_tonumber (_L, -1);
 		    lua_pop (_L, 1);
-
-
-		    /* Resetting the self->stops makes sure that lo remains
-		       smaller than hi between calls. */
-
-		    dJointSetAMotorParam ([self amotor],
-					  dParamLoStop + dParamGroup * j,
-					  -dInfinity);
-		    
-		    dJointSetAMotorParam ([self amotor],
-					  dParamHiStop + dParamGroup * j,
-					  dInfinity);
 		    
 		    dJointSetAMotorParam ([self amotor],
 					  dParamLoStop + dParamGroup * j,
