@@ -19,7 +19,8 @@
 
 #include <lua.h>
 #include <GL/gl.h>
-#include <objc/Object.h>
+
+#include "object.h"
 
 @interface Texture: Object {
     int size[3];
