@@ -17,9 +17,9 @@
 #ifndef _TOP_H_
 #define _TOP_H_
 
-#include "node.h"
+#include "transform.h"
 
-@interface Top: Node {
+@interface Top: Transform {
     GLfloat axis[3], spin;
     double tick;
 }
