@@ -113,7 +113,7 @@ static void recurse (id root)
 
 -(void) transformCustom
 {
-    id child;
+    id child, sister;
     
     /* Calculate the homogenous transform matrix. */
     
@@ -139,7 +139,12 @@ static void recurse (id root)
 
     callhooks (_L, self, self->transform);
 
-    for (child = [self children] ; child ; child = [child sister]) {
+    /* Here we save the sister node beforehand because the child
+       node might be unlinked by its hook function which will
+       result in a prematurely ended traversal. */
+    
+    for (child = [self children] ; child ; child = sister) {
+	sister = [child sister];
 	recurse (child);
     }
 }
