@@ -20,7 +20,24 @@
 #include "node.h"
 
 @interface Transform: Node {
+    double position[3], orientation[9];
+    double translation[3], rotation[9], matrix[16];
+
+    int transform;
 }
+
+-(id) parentTransform;
+-(void) transform;
+-(void) transformCustom;
+-(void) transformAsRoot;
+-(void) transformRelativeTo: (double *) p;
+-(void) transformToTranslation: (double *) r
+                   andRotation: (double *) R;
+-(double *) position;
+-(double *) orientation;
+-(double *) translation;
+-(double *) rotation;
+-(double *) matrix;
 @end
 
 #endif
