@@ -253,6 +253,19 @@
 	    }
 	}
     } else if (!xstrcmp(k, "stops")) {
+	/* Resetting the self->stops makes sure that lo remains
+	   smaller than hi between calls. */
+
+	for(j = 0 ; j < 2 ; j += 1) {
+	    dJointSetUniversalParam ([self joint],
+				     dParamLoStop + dParamGroup * j,
+				     -dInfinity);
+		    
+	    dJointSetUniversalParam ([self joint],
+				     dParamHiStop + dParamGroup * j,
+				     dInfinity);
+	}
+	
         if(lua_istable (_L, 3)) {
 	    for(j = 0 ; j < 2 ; j += 1) {
 		lua_rawgeti (_L, 3, j + 1);
@@ -281,18 +294,6 @@
 		    lua_rawgeti (_L, -1, 3);
 		    bounce[j] = lua_tonumber (_L, -1);
 		    lua_pop (_L, 1);
-
-
-		    /* Resetting the self->stops makes sure that lo remains
-		       smaller than hi between calls. */
-
-		    dJointSetUniversalParam ([self joint],
-					     dParamLoStop + dParamGroup * j,
-					     -dInfinity);
-		    
-		    dJointSetUniversalParam ([self joint],
-					     dParamHiStop + dParamGroup * j,
-					     dInfinity);
 		    
 		    dJointSetUniversalParam ([self joint],
 					     dParamLoStop + dParamGroup * j,
