@@ -39,6 +39,7 @@ static dColliderFn * getCollider (int num)
     };
     
     struct wheeldata *data;
+    int i;
 
     if (!dWheelClass) {
 	struct dGeomClass class = {
@@ -51,9 +52,6 @@ static dColliderFn * getCollider (int num)
 	dWheelClass = dCreateGeomClass (&class);
     }
 
-    self->cylinder = dCreateCylinder (NULL, 0.297, 2 * 0.09);
-    dGeomSetData (self->cylinder, self);
-    
     self->geom = dCreateGeom (dWheelClass);
     dGeomSetData (self->geom, self);
 
@@ -68,6 +66,10 @@ static dColliderFn * getCollider (int num)
     data->airborne = 1;
     
     self->damper = dJointCreateAMotor (_WORLD, NULL);
+
+    self->F_x = 0;
+    self->F_y = 0;
+    self->M_z = 0;
     
     self->F_z0 = 1100;
 
@@ -132,19 +134,16 @@ static dColliderFn * getCollider (int num)
 
     self->resistance = 0.015;
 
+    for (i = 0 ; i < 10 ; i += 1) {
+	data->lambda[i] = 1;
+    }
+    
     [super init];
     [self add: sizeof (list) / sizeof (char *) Properties: list];
 
     return self;
 }
 
--(void) fasten
-{
-    [super fasten];
-
-    dGeomSetBody (self->cylinder, NULL);
-}
-
 -(void) release
 {
     dVector3 a;
@@ -154,29 +153,15 @@ static dColliderFn * getCollider (int num)
     dBodyVectorToWorld (self->body, 0, 1, 0, a);
     
     dBodySetFiniteRotationMode (self->body, 1);
+    dBodySetGyroscopicMode (self->body, 1);
     
     dJointAttach (self->damper, self->body, 0);
     dJointSetAMotorMode (self->damper, dAMotorUser);
     dJointSetAMotorNumAxes (self->damper, 1);
     dJointSetAMotorAxis (self->damper, 0, 1, a[0], a[1], a[2]);
-
-    dGeomSetBody (self->cylinder, self->body);
 }
 
--(void) insertInto: (dSpaceID) new
-{
-    if (self->space) {
-    	dSpaceRemove (self->space, self->cylinder);
-    }
-    
-    if (new) {
-    	dSpaceAdd (new, self->cylinder);
-    }
-
-    [super insertInto: new];
-}
-
--(void)evaluateWithStep: (double)h
+-(void)evaluateWithStep: (double)h andFactors: (double[10])lambda
 {
     double df_z, D_x, E_x, K_xkappa, B_x;
     double gamma_2, D_y, E_y, K_yalpha, B_y, K_ygamma, B_gamma;
@@ -185,12 +170,19 @@ static dColliderFn * getCollider (int num)
     double B_xalpha, B_ykappa, G_ykappa;
     double lambda_r, lambda_t, M_zr, K_2;
     double sigma, V;
+    int i;
 
     struct wheeldata *data;
     
     data = dGeomGetClassData (self->geom);
     V = dLENGTH (dBodyGetLinearVel (self->body));
 
+    /* Combine track and wheel scaling factors. */
+
+    for (i = 0 ; i < 10 ; i += 1) {
+	lambda[i] *= data->lambda[i];
+    }
+
     /* Make sure the load does not vanish completely. */
     
     if (self->F_z < 1e-3) {
@@ -200,18 +192,23 @@ static dColliderFn * getCollider (int num)
     /* if (self->tag == 1) */
     /* printf ("!! %f, %f, %f, %f, %f, %f\n", self->F_z, self->beta, self->beta_1, self->kappa, self->gamma, V); */
 
+    /* printf ("%.2f, %.2f, %.2f, %.2f, %.2f, %.2f, %.2f\n", */
+    /* 	    lambda[0], lambda[1], lambda[2], */
+    /* 	    lambda[3], lambda[4], lambda[5], */
+    /* 	    lambda[6]); */
+
     /* Pure longitudinal slip. */
 
     df_z = (self->F_z - self->F_z0) / self->F_z0;
-    D_x = (self->p_Dx1 + self->p_Dx2 * df_z) * self->F_z;
+    D_x = lambda[0] * (self->p_Dx1 + self->p_Dx2 * df_z) * self->F_z;
     E_x = (self->p_Ex1 + self->p_Ex2 * df_z + self->p_Ex3 * df_z * df_z) * (1 - self->p_Ex4 * sign(self->kappa));
-    K_xkappa = self->F_z * (self->p_Kx1 + self->p_Kx2 * df_z) * exp (self->p_Kx3 * df_z);
+    K_xkappa = lambda[2] * self->F_z * (self->p_Kx1 + self->p_Kx2 * df_z) * exp (self->p_Kx3 * df_z);
     B_x = K_xkappa / (self->C_x * D_x) * self->kappa;
     self->F_x0 = D_x * sin(self->C_x * atan(B_x - E_x * (B_x - atan (B_x))));
 
     /* Sideslip angle lag due to tire deformation. */
 
-    K_yalpha0 = self->p_Ky1 * self->F_z0 * sin (self->p_Ky2 * atan(self->F_z / (self->p_Ky3 * self->F_z0)));
+    K_yalpha0 = lambda[3] * self->p_Ky1 * self->F_z0 * sin (self->p_Ky2 * atan(self->F_z / (self->p_Ky3 * self->F_z0)));
 
     sigma = K_yalpha0 * (self->relaxation[0] +
 			 self->relaxation[1] * V +
@@ -227,18 +224,24 @@ static dColliderFn * getCollider (int num)
     /* Pure lateral slip. */
     
     gamma_2 = self->gamma * self->gamma;
-    K_yalpha = self->p_Ky1 * self->F_z0 * sin (self->p_Ky2 * atan(self->F_z / ((self->p_Ky3 + self->p_Ky4 * gamma_2) * self->F_z0))) / (1 + p_Ky5 * gamma_2);
-    D_y = self->F_z * self->p_Dy1 * exp (self->p_Dy2 * df_z) / (1 + self->p_Dy3 * gamma_2);
+    K_yalpha = lambda[3] * self->p_Ky1 * self->F_z0 * sin (self->p_Ky2 * atan(self->F_z / ((self->p_Ky3 + self->p_Ky4 * gamma_2) * self->F_z0))) / (1 + p_Ky5 * gamma_2);
+    
+    /* The paper by Sharp has a division by the last term but the
+       Pacejka in section 10.6.1 of Tyre and Vehicle Dynamics has a
+       multiplication.  The lisp code for Sharp's paper multiplies as
+       well so we'll assume this is correct. */
+    
+    D_y = lambda[1] * self->F_z * self->p_Dy1 * exp (self->p_Dy2 * df_z) * (1 + self->p_Dy3 * gamma_2);
     E_y = self->p_Ey1 + self->p_Ey2 * gamma_2 + self->p_Ey4 * self->gamma * sign(self->beta_1);
     B_y = K_yalpha / (self->C_y * D_y);
-    K_ygamma = (self->p_Ky6 + self->p_Ky7 * df_z) * self->F_z;
+    K_ygamma = lambda[4] * (self->p_Ky6 + self->p_Ky7 * df_z) * self->F_z;
     B_gamma = K_ygamma / (self->C_gamma * D_y) * self->gamma;
 
     self->F_y0 = D_y * sin (self->C_y * atan (B_y * self->beta_1 - E_y * (B_y * self->beta_1 - atan (B_y * self->beta_1))) + self->C_gamma * atan (B_gamma - self->E_gamma * (B_gamma - atan (B_gamma))));
 
     /* Pure lateral slip for zero camber. */
 
-    D_y0 = self->F_z * self->p_Dy1 * exp (self->p_Dy2 * df_z);
+    D_y0 = lambda[1] * self->F_z * self->p_Dy1 * exp (self->p_Dy2 * df_z);
     B_y0 = K_yalpha0 / (self->C_y * D_y0) * self->beta_1;
 
     F_y00 = D_y0 * sin (self->C_y * atan (B_y0 - self->p_Ey1 * (B_y0 - atan (B_y0))));
@@ -248,11 +251,11 @@ static dColliderFn * getCollider (int num)
     beta_2 = self->beta_1 * self->beta_1;
     S_Hr = (self->q_Hz3 + self->q_Hz4 * df_z) * self->gamma;
     alpha_r = self->beta_1 + S_Hr;
-    B_t = (self->q_Bz1 + self->q_Bz2 * df_z) * (1 + self->q_Bz5 * fabs(self->gamma) + self->q_Bz6 * gamma_2);
-    D_t = self->F_z * (data->radii[1] / self->F_z0) * (self->q_Dz1 + self->q_Dz2 * df_z) * (1 + self->q_Dz3 * fabs(self->gamma) + self->q_Dz4 * gamma_2);
+    B_t = (self->q_Bz1 + self->q_Bz2 * df_z) * (1 + self->q_Bz5 * fabs(self->gamma) + self->q_Bz6 * gamma_2) * lambda[3] / lambda[1];
+    D_t = lambda[5] * self->F_z * (data->radii[1] / self->F_z0) * (self->q_Dz1 + self->q_Dz2 * df_z) * (1 + self->q_Dz3 * fabs(self->gamma) + self->q_Dz4 * gamma_2);
     E_t = (self->q_Ez1 + self->q_Ez2 * df_z + self->q_Ez3 * df_z * df_z) * (1 + self->q_Ez5 * self->gamma * (2 / M_PI) * atan(B_t * self->beta_1 * self->C_t));
-    B_r = self->q_Bz9 + self->q_Bz10 * B_y * self->C_y;
-    D_r = F_z * data->radii[1] * ((self->q_Dz8 + self->q_Dz9 * df_z) * self->gamma + (self->q_Dz10 + self->q_Dz11 * df_z) * self->gamma * fabs(self->gamma)) / sqrt (1 + beta_2);
+    B_r = (self->q_Bz9 + self->q_Bz10 * B_y * self->C_y) * lambda[3] / lambda[1];
+    D_r = lambda[1] * F_z * data->radii[1] * ((self->q_Dz8 + self->q_Dz9 * df_z) * self->gamma * lambda[6] + (self->q_Dz10 + self->q_Dz11 * df_z) * self->gamma * fabs(self->gamma)) / sqrt (1 + beta_2);
 
     /* These are not strictly necessary it seems
        but are included for completeness when plotting. */
@@ -288,7 +291,7 @@ static dColliderFn * getCollider (int num)
     dJointID joint;
     const dReal *r, *v, *omega;
     dVector3 p;
-    dReal V_l, l;
+    dReal V_l, r_e;
 
     data = dGeomGetClassData (self->geom);
 
@@ -311,21 +314,29 @@ static dColliderFn * getCollider (int num)
     p[1] = data->contact.pos[1] + data->contact.normal[1] * data->contact.depth;
     p[2] = data->contact.pos[2] + data->contact.normal[2] * data->contact.depth;
 
-    l = (p[0] - r[0]) * data->radial[0] +
+    r_e = (p[0] - r[0]) * data->radial[0] +
 	(p[1] - r[1]) * data->radial[1] +
 	(p[2] - r[2]) * data->radial[2];
     
     self->gamma = asin(dDOT(data->axial, data->contact.normal));
 
-    /* beta is the tangent of the slip angle which is the input to the
-       magic formula according to: Tyre and vehicle dynamics pp. 173 */
+    /* The slip ratio kappa is the ratio of the difference between the
+       spin velocity of the driven tire (V_l / r_e) and that of the
+       free-rolling tire (omega) to the spin velocity of the
+       free-rolling tire (according to SAE J670e). */
+    
+    self->kappa = (dDOT (omega, data->axial) * r_e - V_l) / fabs(V_l);
+
+    /* Here beta is the tangent of the slip angle which is the input
+       to the magic formula according to: Tyre and vehicle dynamics
+       pp. 173 */
     
-    self->kappa = (V_l - dDOT (omega, data->axial) * l) / fabs(V_l);
     self->beta = dDOT (v, data->lateral) / fabs(V_l);
 
-    /* if (self->tag == 1) { */
-    /* 	printf ("%f, %f, %f, %f\n", self->kappa, self->beta, self->F_x, self->F_y); */
-    /* } */
+    if (self->tag == 1) {
+    	/* printf ("%f, %f, %f, %f\n", self->kappa, self->beta, self->F_x, self->F_y); */
+	/* printf ("%f\n", r_e); */
+    }
 
     /* Take care of singularities. */
     
@@ -340,63 +351,137 @@ static dColliderFn * getCollider (int num)
     /* printf ("%f, %f, %f\n", V_l, self->kappa, self->beta); */
 
     if (!data->airborne) {
-	/* printf ("** %f, %f, %f, %f, %f, %f\n", self->F_x0, self->F_x, self->F_y0, self->F_y, self->M_z0, self->M_z); */
-	
-	/* Create the contact joint. */
-	
-	contact.geom = data->contact;
+	double lambda[10] = {1, 1, 1, 1, 1, 1, 1, 1, 1, 1};
+	int airborne = 0;
+	int i, h_0, h_1;
 
-	/* If the vehicle velocity is very low switch
-	   to normal ode friction as its more stable.*/
+	/* Call the collision callback for the pair
+	   of geoms and get the returned parameters. */
 	
-	contact.surface.mode = dContactSoftERP | dContactSoftCFM;
-	if (V_l < 1e-1) {
-	    contact.surface.mode |= dContactApprox1;
-	    contact.surface.mu = 1;
+	h_0 = lua_gettop (_L);
+		    
+	lua_getfield(_L, LUA_GLOBALSINDEX, "dynamics");
+	lua_getfield(_L, -1, "collision");
+	lua_replace (_L, -2);
+	
+	if (!lua_isnil(_L, -1)) {
+	    lua_getfield (_L, LUA_REGISTRYINDEX, "userdata");
+
+	    /* Get the userdata. */
+	    
+	    lua_pushlightuserdata (_L, dGeomGetData(data->contact.g1));
+	    lua_gettable (_L, -2);
+
+	    lua_pushlightuserdata (_L, dGeomGetData(data->contact.g2));
+	    lua_gettable (_L, -3);
+
+	    lua_pushvalue (_L, -4);
+
+	    /* If it's a single function encapsulate it
+	       in a table to unify the approach. */
+
+	    if (!lua_istable (_L, -1)) {
+		lua_newtable (_L);
+		lua_insert (_L, -2);
+		lua_rawseti (_L, -2, 1);
+	    }
+
+	    /* Call all bound functions. */
+		    
+	    lua_pushnil (_L);
+	    h_1 = lua_gettop (_L);
+
+	    while (lua_next (_L, -2) != 0) {
+		lua_pushvalue (_L, -5);
+		lua_pushvalue (_L, -5);
+
+		luaX_call (_L, 2, LUA_MULTRET);
+
+		if (lua_type (_L, h_1 + 1) == LUA_TNUMBER) {
+		    airborne = lua_tointeger (_L, h_1 + 1) == 0;
+		}
+
+		for (i = 0 ; i < 10 ; i += 1) {
+		    if (lua_type (_L, h_1 + 2 + i) == LUA_TNUMBER) {
+			lambda[i] = lua_tonumber (_L, h_1 + 2 + i);
+		    }
+		}
+
+		lua_settop (_L, h_1);
+	    }
+	}
+		    
+	lua_settop (_L, h_0);
+
+	/* Recheck in case the user requested not contact. */
+    
+	if (!airborne) {
+	    /* Create the contact joint. */
+	
+	    contact.geom = data->contact;
+
+	    /* If the vehicle velocity is very low switch
+	       to normal ode friction as it is more stable.*/
+	
+	    contact.surface.mode = dContactSoftERP | dContactSoftCFM;
+	    if (fabs(V_l) < 1e-1) {
+		contact.surface.mode |= dContactApprox1;
+		contact.surface.mu = 1;
+	    } else {
+		contact.surface.mu = 0;
+	    }
+
+	    {
+		dReal k_s, k_d;
+	    
+		k_s = lambda[8] * data->elasticity[0];
+		k_d = lambda[9] * data->elasticity[1];
+	
+		contact.surface.soft_cfm = 1.0 / (h * k_s + k_d);
+		contact.surface.soft_erp = h * k_s / (h * k_s + k_d);
+	    }
+	
+	    joint = dJointCreateContact (_WORLD, _GROUP, &contact);
+	    dJointSetFeedback (joint, &feedback);
+	    dJointAttach (joint,
+			  dGeomGetBody (data->contact.g1),
+			  dGeomGetBody (data->contact.g2));
+
+	    /* if (self->tag == 1) { */
+	    /*     fprintf (stderr, "%f, %f, %f\n", */
+	    /* 	     t, contact.geom.pos[2], contact.geom.depth); */
+	    /* } */
+
+	    if (fabs(V_l) > 1e-1) {
+		[self evaluateWithStep: h andFactors: lambda];
+
+		dBodyAddForceAtPos (self->body,
+				    data->longitudinal[0] * self->F_x,
+				    data->longitudinal[1] * self->F_x,
+				    data->longitudinal[2] * self->F_x,
+				    p[0], p[1], p[2]);
+
+		dBodyAddForceAtPos (self->body,
+				    -data->lateral[0] * self->F_y,
+				    -data->lateral[1] * self->F_y,
+				    -data->lateral[2] * self->F_y,
+				    p[0], p[1], p[2]);
+
+		dBodyAddTorque (self->body,
+				-data->contact.normal[0] * self->M_z,
+				-data->contact.normal[1] * self->M_z,
+				-data->contact.normal[2] * self->M_z);
+
+		/* Set the rolling resistance torque. */
+	
+		dJointSetAMotorParam (self->damper, dParamFMax,
+				      self->F_z * lambda[7] * self->resistance * r_e);
+	    }
 	} else {
-	    contact.surface.mu = 0;
+	    dJointSetAMotorParam (self->damper, dParamFMax, 0);
 	}
-
-	contact.surface.soft_cfm = 1.0 / (h * data->elasticity[0] +
-					  data->elasticity[1]);
-	contact.surface.soft_erp = h * data->elasticity[0] /
-	    (h * data->elasticity[0] + data->elasticity[1]);
-
-	joint = dJointCreateContact (_WORLD, _GROUP, &contact);
-	dJointSetFeedback (joint, &feedback);
-	dJointAttach (joint,
-		      dGeomGetBody (data->contact.g1),
-		      dGeomGetBody (data->contact.g2));
-
-	if (V_l > 1e-1) {
-	    [self evaluateWithStep: h];
-	
-	    dBodyAddForceAtPos (self->body,
-				-data->longitudinal[0] * self->F_x,
-				-data->longitudinal[1] * self->F_x,
-				-data->longitudinal[2] * self->F_x,
-				p[0], p[1], p[2]);
-
-	    dBodyAddForceAtPos (self->body,
-				-data->lateral[0] * self->F_y,
-				-data->lateral[1] * self->F_y,
-				-data->lateral[2] * self->F_y,
-				p[0], p[1], p[2]);
-
-	    dBodyAddTorque (self->body,
-			    -data->contact.normal[0] * self->M_z,
-			    -data->contact.normal[1] * self->M_z,
-			    -data->contact.normal[2] * self->M_z);
-
-	    /* Set the rolling resistance torque. */
-	
-	    dJointSetAMotorParam (self->damper, dParamFMax,
-				  self->F_z * self->resistance * l);
-	}
-    } else {
-	dJointSetAMotorParam (self->damper, dParamFMax, 0);
     }
-
+    
     [super stepBy: h at: t];
 }
 
@@ -636,10 +721,6 @@ static dColliderFn * getCollider (int num)
     } else if (!xstrcmp(k, "resistance")) {
 	lua_pushnumber (_L, self->resistance);
     } else if (!xstrcmp(k, "state")) {
-	struct wheeldata *data;
-
-	data = dGeomGetClassData (self->geom);
-
 	lua_newtable (_L);
 
 	lua_pushnumber (_L, self->gamma);
@@ -671,6 +752,17 @@ static dColliderFn * getCollider (int num)
  
 	lua_pushnumber (_L, self->M_z0);
 	lua_rawseti (_L, -2, 10);
+    } else if (!xstrcmp(k, "scaling")) {
+	struct wheeldata *data;
+
+	data = dGeomGetClassData (self->geom);
+
+	lua_newtable(_L);
+	
+	for (i = 0 ; i < 10 ; i += 1) {
+	    lua_pushnumber (_L, data->lambda[i]);
+	    lua_rawseti (_L, -2, i + 1);
+	}
     } else {
 	[super get];
     }
@@ -706,10 +798,6 @@ static dColliderFn * getCollider (int num)
 	lua_rawgeti (_L, 3, 2);
 	data->radii[1] = lua_tonumber (_L, -1);
 
-	dGeomCylinderSetParams (self->cylinder,
-				data->radii[0] + data->radii[1],
-				2 * data->radii[1]);
-
 	lua_pop (_L, 2);
     } else if (!xstrcmp(k, "longitudinal")) {
 	/* Parameters for pure and combined
@@ -898,6 +986,8 @@ static dColliderFn * getCollider (int num)
     } else if (!xstrcmp(k, "resistance")) {
 	self->resistance = lua_tonumber (_L, 3);
     } else if (!xstrcmp(k, "state")) {
+	double one[10] = {1, 1, 1, 1, 1, 1, 1, 1, 1, 1};
+	
 	lua_rawgeti(_L, 3, 1);
 	self->gamma = lua_tonumber (_L, -1);
 	
@@ -910,29 +1000,17 @@ static dColliderFn * getCollider (int num)
 	lua_rawgeti(_L, 3, 4);
 	self->F_z = lua_tonumber (_L, -1);
 
-	[self evaluateWithStep: 0];
-    } else if (!xstrcmp(k, "position")) {
-    	[super set];
-	
-    	dGeomSetPosition (self->cylinder,
-    			  self->position[0],
-    			  self->position[1],
-    			  self->position[2]);
-    } else if (!xstrcmp(k, "orientation")) {
-    	dMatrix3 R;
-    	int i, j;
-	    
-    	[super set];
+	[self evaluateWithStep: 0 andFactors: one];
+    } else if (!xstrcmp(k, "scaling")) {
+	struct wheeldata *data;
 
-    	dRSetIdentity(R);
+	data = dGeomGetClassData (self->geom);
 
-    	for(i = 0 ; i < 3 ; i += 1) {
-    	    for(j = 0 ; j < 3 ; j += 1) {
-    		R[i * 4 + j] = self->orientation[i * 3 + j];
-    	    }
-    	}
-
-    	dGeomSetRotation (self->cylinder, R);
+	for (i = 0 ; i < 10 ; i += 1) {
+	    lua_rawgeti (_L, 3, i + 1);
+	    data->lambda[i] = lua_tonumber (_L, -1);
+	    lua_pop (_L, 1);
+	}
     } else {
 	[super set];
     }
@@ -998,7 +1076,20 @@ static dColliderFn * getCollider (int num)
 	glMatrixMode (GL_MODELVIEW);
 	glPushMatrix();
 
-	glMultMatrixd ([self matrix]);
+	{
+	    const dReal *p, *R;
+	    double T[16];
+	    
+	    p = dBodyGetPosition (self->body);
+	    R = dBodyGetRotation (self->body);
+
+	    T[0] = R[0]; T[1] = R[4]; T[2] = R[8]; T[3] = 0;
+	    T[4] = R[1]; T[5] = R[5]; T[6] = R[9]; T[7] = 0;
+	    T[8] = R[2]; T[9] = R[6]; T[10] = R[10]; T[11] = 0;
+	    T[12] = p[0]; T[13] = p[1]; T[14] = p[2]; T[15] = 1;
+	    
+	    glMultMatrixd(T);
+	}
 
 	glBegin (GL_LINE_STRIP);
 
