@@ -17,19 +17,17 @@
 #ifndef _WIDGET_H_
 #define _WIDGET_H_
 
-#include "node.h"
+#include "transform.h"
 
-@interface Widget: Node {
-    GLfloat color[4];
-    float requested[2], allocated[2], minimum[2];
-    GLfloat padding[2];
+@interface Widget: Transform {
+    double color[4], background[4], padding[2];
+    double allocated[2], minimum[2];
     int align[2];
 }
 
--(GLfloat *)color;
--(float) opacity;
--(float) width;
--(float) height;
+-(double) measureWidth;
+-(double) measureHeight;
+-(double *) allocated;
 
 @end
 
