@@ -138,7 +138,7 @@ int luaopen_widgets (lua_State *L)
     
     Class nodes[] = {
 	[Display class], [Frame class], [Column class], [Row class],
-	[Annotation class], [Clock class], [Layout class], [Screen class]
+	[Annotation class], [Clock class], [Layout class], [Screen class],
     };	
 
     lua_newtable (L);
